import 'package:flutter/material.dart';
import 'package:nb_utils/nb_utils.dart';

enum TextFieldType { EMAIL, PASSWORD, NAME, ADDRESS, USERNAME }

class AppTextField extends StatefulWidget {
  final TextEditingController controller;
  final FocusNode focus;
  final FormFieldValidator<String> validator;
  final TextFieldType textFieldType;
  final bool isPassword;
  final bool isRequired;
  final TextCapitalization textCapitalization;
  final TextInputAction textInputAction;
  final Function(String) onFieldSubmitted;
  final Function(String) onChanged;
  final FocusNode nextFocus;
  final TextStyle textStyle;
  final TextAlign textAlign;
  final int maxLines;
  final int minLines;
  final InputDecoration decoration;
  final bool enabled;
  final Color cursorColor;
  bool isPasswordVisible;
  final Widget suffix;

  AppTextField({
    @required this.controller,
    @required this.textFieldType,
    @required this.decoration,
    this.focus,
    this.validator,
    this.isPassword,
    this.isRequired,
    this.textCapitalization,
    this.textInputAction,
    this.onFieldSubmitted,
    this.nextFocus,
    this.textStyle,
    this.textAlign,
    this.maxLines,
    this.minLines,
    this.enabled,
    this.onChanged,
    this.cursorColor,
    this.suffix,
    this.isPasswordVisible = false,
  });

  @override
  _AppTextFieldState createState() => _AppTextFieldState();
}

class _AppTextFieldState extends State<AppTextField> {
  @override
  Widget build(BuildContext context) {
    return TextFormField(
      controller: widget.controller,
      focusNode: widget.focus,
      obscureText: widget.textFieldType == TextFieldType.PASSWORD && !widget.isPasswordVisible,
      validator: widget.isRequired.validate(value: true)
          ? widget.validator ?? widget.textFieldType == TextFieldType.EMAIL
              ? (s) {
                  if (s.trim().isEmpty) return errorThisFieldRequired;
                  if (!s.trim().validateEmail()) return 'Email is invalid';
                  return null;
                }
              : widget.textFieldType == TextFieldType.PASSWORD
                  ? (s) {
                      if (s.trim().isEmpty) return errorThisFieldRequired;
                      if (s.trim().length < passwordLengthGlobal) return '';
                      return null;
                    }
                  : widget.textFieldType == TextFieldType.NAME || widget.textFieldType == TextFieldType.USERNAME
                      ? (s) {
                          if (s.trim().isEmpty) return errorThisFieldRequired;
                          return null;
                        }
                      : null
          : null,
      textCapitalization: widget.textCapitalization ?? widget.textFieldType == TextFieldType.EMAIL ? TextCapitalization.none : TextCapitalization.words,
      textInputAction: widget.textInputAction ?? TextInputAction.next,
      onFieldSubmitted: (s) {
        if (widget.nextFocus != null) FocusScope.of(context).requestFocus(widget.nextFocus);

        if (widget.onFieldSubmitted != null) widget.onFieldSubmitted.call(s);
      },
      style: widget.textStyle,
      textAlign: widget.textAlign ?? TextAlign.left,
      maxLines: widget.maxLines ?? 1,
      minLines: widget.minLines ?? 1,
      keyboardType: widget.textFieldType == TextFieldType.EMAIL
          ? TextInputType.emailAddress
          : widget.textFieldType == TextFieldType.ADDRESS
              ? TextInputType.multiline
              : widget.textFieldType == TextFieldType.PASSWORD
                  ? TextInputType.visiblePassword
                  : TextInputType.text,
      decoration: widget.decoration.copyWith(
        suffix: widget.textFieldType == TextFieldType.PASSWORD
            ? widget.suffix ??
                Icon(widget.isPasswordVisible ? Icons.visibility : Icons.visibility_off).onTap(() {
                  widget.isPasswordVisible = !widget.isPasswordVisible;

                  setState(() {});
                })
            : null,
      ),
      enabled: widget.enabled,
      onChanged: widget.onChanged,
      cursorColor: widget.cursorColor,
    );
  }
}
