import 'package:flutter/material.dart';
import 'package:nb_utils/nb_utils.dart';

class SettingItemWidget extends StatelessWidget {
  static String tag = '/SettingItemWidget';

  final String title;
  final String subTitle;
  final Widget leading;
  final Widget trailing;
  final TextStyle titleTextStyle;
  final TextStyle subTitleTextStyle;
  final Function onTap;
  final EdgeInsets padding;
  final Color titleTextColor;
  final Color subTitleTextColor;

  SettingItemWidget({
    @required this.title,
    @required this.onTap,
    this.subTitle,
    this.leading,
    this.trailing,
    this.titleTextStyle,
    this.subTitleTextStyle,
    this.padding,
    this.titleTextColor,
    this.subTitleTextColor,
  })  : assert(title != null),
        assert(title.validate().isNotEmpty),
        assert(onTap != null);

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: padding ?? EdgeInsets.all(16),
      child: Row(
        children: [
          leading ?? SizedBox(),
          10.width.visible(leading != null),
          Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Text(title.validate(), style: titleTextStyle ?? boldTextStyle(color: titleTextColor ?? textPrimaryColorGlobal), maxLines: 1, overflow: TextOverflow.ellipsis),
              4.height.visible(subTitle.validate().isNotEmpty),
              Text(
                subTitle.validate(),
                style: subTitleTextStyle ?? secondaryTextStyle(color: subTitleTextColor ?? textSecondaryColorGlobal),
                maxLines: 2,
                overflow: TextOverflow.ellipsis,
              ).visible(subTitle.validate().isNotEmpty),
            ],
          ).expand(),
          8.width.visible(trailing != null),
          trailing ?? SizedBox(),
        ],
      ),
    ).onTap(onTap);
  }
}
